#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

//#include "hiddescriptor.h"
#include "usbdrv.h"

#include "../../include/phimatics_cmds.h" // common for firmware & externals


// ==============================================================================
// Globals
// ------------------------------------------------------------------------------
void setLed(void);
void setAllLed(void);

static uint8_t mode;
static uint8_t usb_reply[1];

static uint8_t datenbuffer[3]; // 3 bytes - channel, 4bits pwm, 8 bits rest of pwm
static uint8_t currentPosition, bytesRemaining;

// ------------------------------------------------------------------------------
// - usbFunctionSetup
// - see: http://vusb.wikidot.com/driver-api
// ------------------------------------------------------------------------------
usbMsgLen_t usbFunctionSetup(uchar data[8]) {
    usbRequest_t    *rq = (void *)data;

    if(rq->bRequest == PHIMATICS_CMD_POLL) {
        usbMsgLen_t len = sizeof(usb_reply); // we return up to 64 bytes
        if(len > rq->wLength.word)           // if the host requests less than we have
            len = rq->wLength.word;          // return only the amount requested
        usbMsgPtr = usb_reply;                  // tell driver where the buffer starts
        return len;

        
    } else if(rq->bRequest == PHIMATICS_CMD_ALLOFF) {
        setAllOff();

    } else if(rq->bRequest == PHIMATICS_CMD_ALLON) {
        setAllOn();

    } else {
        mode = rq->bRequest;
        currentPosition = 0;                // initialize position index
        bytesRemaining = rq->wLength.word;  // store the amount of data requested
        if(bytesRemaining > sizeof(datenbuffer)) // limit to buffer size
            bytesRemaining = sizeof(datenbuffer);
        return USB_NO_MSG;        // tell driver to use usbFunctionWrite
    }

    return 0;
}

//do not forget to set #define USB_CFG_IMPLEMENT_FN_WRITE 1 in usbconfig.h
uchar usbFunctionWrite(uchar *data, uchar len)
{
    uchar i;
    if(len > bytesRemaining)                // if this is the last incomplete chunk
        len = bytesRemaining;               // limit to the amount we can store
    bytesRemaining -= len;
    for(i = 0; i < len; i++)
        datenbuffer[currentPosition++] = data[i];

    switch(mode) {
    	case PHIMATICS_CMD_ALLLED:
    		setAllLed();
    	break;
    	case PHIMATICS_CMD_LED:
    		setLed();
    	break;
    }

    return bytesRemaining == 0;             // return 1 if we have all data
}

void setAllLed(void) {
	uint16_t pwm = datenbuffer[0] << 8 | datenbuffer[1]; // because were using 2 full bytes for 12bits...
}

void setLed(void) {
	uint8_t channel = datenbuffer[0];
	uint16_t pwm = datenbuffer[1] << 8 | datenbuffer[2]; // because were using 2 full bytes for 12bits...
}

/*---------------------------------------------------------------------------*/
/* usbConf                                                              */
/*---------------------------------------------------------------------------*/
static void hardwareInit(void)
{
	uchar i, j;

	/* activate pull-ups except on USB lines */
	USB_CFG_IOPORT = (uchar) ~ ((1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT));
	
	/* all pins input except USB (-> USB reset) */
	#ifdef USB_CFG_PULLUP_IOPORT	/* use usbDeviceConnect()/usbDeviceDisconnect() if available */
		USBDDR = 0;		/* we do RESET by deactivating pullup */
		usbDeviceDisconnect();
	#else
		USBDDR = (1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT);
	#endif

	j = 0;
	while (--j) {		/* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while (--i);	/* delay >10ms for USB reset */
	}
	
	#ifdef USB_CFG_PULLUP_IOPORT
		usbDeviceConnect();
	#else
		USBDDR = 0;		/*  remove USB reset condition */
	#endif
}


// ==============================================================================
// - main
// ------------------------------------------------------------------------------
int __attribute__((noreturn)) main(void)
{
	hardwareInit();
	usbInit();
	sei();

	for(;;) {

		usbPoll();
		usb_reply[0] = 14;	

	}
}

